@@ -1,9 +1,10 @@
 import logging
-
+from flask import g
 from flask_wtf import Form
 from wtforms import (BooleanField, TextField,
                        TextAreaField, IntegerField, FloatField, DateField, DateTimeField)
 
+
 from wtforms.ext.sqlalchemy.fields import QuerySelectMultipleField, QuerySelectField
 
 from wtforms import validators
@@ -20,12 +21,26 @@ from .upload import (BS3FileUploadFieldWidget,
                     FileUploadField,
                     ImageUploadField)
 from .validators import Unique
+from functools import partial
 
-
+try:
+    from wtforms.fields.core import _unset_value as unset_value
+except:
+    from wtforms.utils import unset_value
+    
 
 log = logging.getLogger(__name__)
 
 
+def get_cascade_value_helper(col_name=""):
+    from flask import request
+
+    if not col_name:
+        return None
+    else:
+        obj = getattr(g, '_current_form_obj', None)
+        return getattr(obj, col_name, None)
+
 class FieldConverter(object):
     conversion_table = (('is_image', ImageUploadField, BS3ImageUploadFieldWidget),
                         ('is_file', FileUploadField, BS3FileUploadFieldWidget),
@@ -68,28 +83,23 @@ class GeneralModelConverter(object):
     def __init__(self, datamodel):
         self.datamodel = datamodel
 
-    def _get_validators(self, colname, validators_columns):
-        if colname in validators_columns:
-            return validators_columns[colname]
-        else:
-            return []
+    @staticmethod
+    def _get_validators(col_name, validators_columns):
+        return validators_columns.get(col_name, [])
 
-    def _get_description(self, colname, description_columns):
-        if colname in description_columns:
-            return description_columns[colname]
-        else:
-            return ""
+    @staticmethod
+    def _get_description(col_name, description_columns):
+        return description_columns.get(col_name, "")
 
-    def _get_label(self, colname, label_columns):
-        if colname in label_columns:
-            return label_columns[colname]
-        else:
-            return ""
+    @staticmethod
+    def _get_label(col_name, label_columns):
+        return label_columns.get(col_name, "")
 
     def _get_func_related_query(self, col_name, filter_rel_fields):
         if filter_rel_fields:
             for filter_rel_field in filter_rel_fields:
                 if filter_rel_field[0] == col_name:
+                    # filter_rel_field[1] if the Data Interface class with model
                     sqla = filter_rel_field[1]
                     _filters = self.datamodel.get_filters().add_filter_list(sqla, filter_rel_field[2])
                     return lambda: sqla.query(_filters)[1]
@@ -97,9 +107,58 @@ class GeneralModelConverter(object):
         return lambda: self.datamodel.session.query(rel_model)
 
 
+    def _get_func_cascade_query(self, col_name, filter_rel_fields, cascade_rel_fields, form_props):
+        for cascade_rel_field in cascade_rel_fields:
+            if col_name == cascade_rel_field[1]:
+                sqla = cascade_rel_field[2]
+
+                filter_item = list(cascade_rel_field[3])
+                related_model = sqla._get_related_model(filter_item[0])[0]
+                filter_item[2] = partial(get_cascade_value_helper, col_name=filter_item[2])
+                
+                _filters = self.datamodel.get_filters().add_filter_list(sqla, [filter_item])
+                return lambda: sqla.query(_filters)[1]
+
+
+    def is_master_cascade_field(self, col_name, cascade_rel_fields):
+        """
+            Checks if field (col_name) is a master field on a
+            cascade related fields definition.
+        """
+        if cascade_rel_fields:
+            for cascade_rel_field in cascade_rel_fields:
+                if col_name == cascade_rel_field[0]:
+                    return True
+        return False
+
+    def is_slave_cascade_field(self, col_name, cascade_rel_fields):
+        """
+            Checks if field (col_name) is a master field on a
+            cascade related fields definition.
+        """
+        if cascade_rel_fields:
+            for cascade_rel_field in cascade_rel_fields:
+                if col_name == cascade_rel_field[1]:
+                    return True
+        return False
+
+
     def _convert_many_to_one(self, col_name, label, description,
-                             lst_validators, filter_rel_fields, form_props):
-        query_func = self._get_func_related_query(col_name, filter_rel_fields)
+                             lst_validators, filter_rel_fields,
+                             cascade_rel_fields, form_props):
+        """
+            Creates a WTForm field for many to one related fields,
+            will use a Select box based on a query. Will only
+            work with SQLAlchemy interface.
+        """
+        if self.is_slave_cascade_field(col_name, cascade_rel_fields):
+            query_func = self._get_func_cascade_query(col_name, filter_rel_fields, cascade_rel_fields, form_props)
+        else:
+            query_func = self._get_func_related_query(col_name, filter_rel_fields)
+        extra_classes = None
+        if self.is_master_cascade_field(col_name, cascade_rel_fields):
+            # it's master field get's css class for on change post.
+            extra_classes = 'json_select2'
         allow_blank = True
         col = self.datamodel.get_relation_fk(col_name)
         if not col.nullable:
@@ -113,11 +172,12 @@ class GeneralModelConverter(object):
                              query_factory=query_func,
                              allow_blank=allow_blank,
                              validators=lst_validators,
-                             widget=Select2Widget())
+                             widget=Select2Widget(extra_classes=extra_classes))
         return form_props
 
     def _convert_many_to_many(self, col_name, label, description,
-                              lst_validators, filter_rel_fields, form_props):
+                              lst_validators, filter_rel_fields,
+                              cascade_rel_fields, form_props):
         query_func = self._get_func_related_query(col_name, filter_rel_fields)
         allow_blank = True
         form_props[col_name] = \
@@ -141,20 +201,28 @@ class GeneralModelConverter(object):
         form_props[col_name] = fc.convert()
         return form_props
 
-    def _convert_prop(self, col_name, label, description, lst_validators, filter_rel_fields, form_props):
+    def _convert_prop(self, col_name,
+                      label, description,
+                      lst_validators, filter_rel_fields,
+                      cascade_rel_fields,
+                      form_props):
         if self.datamodel.is_relation(col_name):
             if self.datamodel.is_relation_many_to_one(col_name) or \
                     self.datamodel.is_relation_one_to_one(col_name):
                 return self._convert_many_to_one(col_name, label,
                                                  description,
                                                  lst_validators,
-                                                 filter_rel_fields, form_props)
+                                                 filter_rel_fields,
+                                                 cascade_rel_fields,
+                                                 form_props)
             elif self.datamodel.is_relation_many_to_many(col_name) or \
                     self.datamodel.is_relation_one_to_many(col_name):
                 return self._convert_many_to_many(col_name, label,
                                                   description,
                                                   lst_validators,
-                                                  filter_rel_fields, form_props)
+                                                  filter_rel_fields,
+                                                  cascade_rel_fields,
+                                                 form_props)
             else:
                 log.warning("Relation {0} not supported".format(col_name))
         else:
@@ -164,7 +232,7 @@ class GeneralModelConverter(object):
 
     def create_form(self, label_columns={}, inc_columns=[],
                     description_columns={}, validators_columns={},
-                    extra_fields={}, filter_rel_fields=None):
+                    extra_fields={}, filter_rel_fields=None, cascade_rel_fields=None):
         form_props = {}
         for col_name in inc_columns:
             if col_name in extra_fields:
@@ -174,14 +242,18 @@ class GeneralModelConverter(object):
                 self._convert_prop(col_name, self._get_label(col_name, label_columns),
                                    self._get_description(col_name, description_columns),
                                    self._get_validators(col_name, validators_columns),
-                                   filter_rel_fields, form_props)
+                                   filter_rel_fields, cascade_rel_fields, form_props)
         return type('DynamicForm', (DynamicForm,), form_props)
 
 
 class DynamicForm(Form):
+
     @classmethod
     def refresh(self, obj=None):
+        g._current_form_obj = obj                           
         form = self(obj=obj)
         return form
 
 
+
+        
