@@ -1,7 +1,7 @@
 import logging
-from flask import render_template, flash, redirect, send_file
+from flask import render_template, flash, redirect, send_file, jsonify
 from .filemanager import uuid_originalname
-from .security.decorators import has_access
+from .security.decorators import has_access, permission_name
 from .widgets import FormWidget, GroupFormListWidget, ListMasterWidget
 from .baseviews import expose, BaseView, BaseModelView, BaseCRUDView
 from .models.group import GroupByProcessData
@@ -22,8 +22,13 @@ class IndexView(BaseView):
 
     @expose('/')
     def index(self):
+        self.update_redirect()
         return render_template(self.index_template, appbuilder=self.appbuilder)
 
+    @expose('/back')
+    def back(self):
+        return redirect(self.get_redirect())
+        
 
 class SimpleFormView(BaseView):
     """
@@ -67,6 +72,7 @@ class SimpleFormView(BaseView):
 
         self.form_get(form)
         widgets = self._get_edit_widget(form=form)
+        self.update_redirect()
         return render_template(self.form_template,
                                title=self.form_title,
                                widgets=widgets,
@@ -87,7 +93,7 @@ class SimpleFormView(BaseView):
 
         if form.validate_on_submit():
             self.form_post(form)
-            return redirect(self._get_redirect())
+            return redirect(self.get_redirect())
         else:
             widgets = self._get_edit_widget(form=form)
             return render_template(
@@ -158,6 +164,8 @@ class ModelView(BaseCRUDView):
                                related_views=self._related_views)
 
 
+
+
     """
     ---------------------------
             ADD
@@ -170,13 +178,30 @@ class ModelView(BaseCRUDView):
 
         widget = self._add()
         if not widget:
-            return redirect(self._get_redirect())
+            return redirect(self.get_redirect())
         else:
             return render_template(self.add_template,
                                    title=self.add_title,
                                    widgets=widget,
                                    appbuilder=self.appbuilder)
 
+    @has_access
+    @permission_name('add')
+    @expose('/add_post', methods=['POST'])
+    def add_post(self):
+        get_filter_args(self._filters)
+        exclude_cols = self._filters.get_relation_cols()
+
+        item = self.datamodel.obj()
+        form = self.add_form.refresh()
+        form.populate_obj(item)
+        widget = self._get_add_widget(form=form, exclude_cols=exclude_cols)
+        return render_template(self.add_template,
+                                   title=self.add_title,
+                                   widgets=widget,
+                                   appbuilder=self.appbuilder)
+
+
     """
     ---------------------------
             EDIT
@@ -188,7 +213,7 @@ class ModelView(BaseCRUDView):
     def edit(self, pk):
         widgets = self._edit(pk)
         if not widgets:
-            return redirect(self._get_redirect())
+            return redirect(self.get_redirect())
         else:
             return render_template(self.edit_template,
                                    title=self.edit_title,
@@ -207,7 +232,35 @@ class ModelView(BaseCRUDView):
     @has_access
     def delete(self, pk):
         self._delete(pk)
-        return redirect(self._get_redirect())
+        return redirect(self.get_redirect())
+
+    
+    @has_access
+    @permission_name('list')
+    @expose('/json')
+    def json(self):
+        if get_order_args().get(self.__class__.__name__):
+            order_column, order_direction = get_order_args().get(self.__class__.__name__)
+        else:
+            order_column, order_direction = '', ''
+        page = get_page_args().get(self.__class__.__name__)
+        page_size = get_page_size_args().get(self.__class__.__name__)
+        get_filter_args(self._filters)
+        if not order_column and self.base_order:
+            order_column, order_direction = self.base_order
+        joined_filters = self._filters.get_joined_filters(self._base_filters)
+        count, lst = self.datamodel.query(joined_filters, order_column, order_direction, page=page, page_size=page_size)
+        result = [item.to_json() for item in lst]
+        return jsonify(label_columns=self.label_columns,
+                        include_columns=self.list_columns,
+                        order_columns=self.order_columns,
+                        page=page,
+                        page_size=page_size,
+                        count=count,
+                        modelview_name=self.__class__.__name__,
+                        result=result)
+
+
 
 
     @expose('/download/<string:filename>')
@@ -317,7 +370,7 @@ class CompactCRUDMixin(BaseCRUDView):
             self._session_form_widget = widgets.get('add')
             self._session_form_action = request.url
             self._session_form_title = self.add_title
-            return redirect(self._get_redirect())
+            return redirect(self.get_redirect())
 
 
     @expose('/edit/<pk>', methods=['GET', 'POST'])
@@ -328,12 +381,12 @@ class CompactCRUDMixin(BaseCRUDView):
             self._session_form_action = ''
             self._session_form_widget = None
 
-            return redirect(self._get_redirect())
+            return redirect(self.get_redirect())
         else:
             self._session_form_widget = widgets.get('edit')
             self._session_form_action = request.url
             self._session_form_title = self.edit_title
-            return redirect(self._get_redirect())
+            return redirect(self.get_redirect())
 
 """
     This is for retro compatibility
